@@ -8,12 +8,13 @@ set -ex
 
 # Generate artifacts for release
 mk_artifacts() {
+    CARGO="$(builder)"
     if is_arm; then
-        cargo build --target "$TARGET" --release
+        "$CARGO" build --target "$TARGET" --release
     else
         # Technically, MUSL builds will force PCRE2 to get statically compiled,
         # but we also want PCRE2 statically build for macOS binaries.
-        PCRE2_SYS_STATIC=1 cargo build --target "$TARGET" --release --features 'pcre2'
+        PCRE2_SYS_STATIC=1 "$CARGO" build --target "$TARGET" --release --features 'pcre2'
     fi
 }
 
