@@ -9,7 +9,8 @@
 // is where we read clap's configuration from the end user's arguments and turn
 // it into a ripgrep-specific configuration type that is not coupled with clap.
 
-use clap::{self, App, AppSettings};
+use clap::{self, App, AppSettings, crate_authors, crate_version};
+use lazy_static::lazy_static;
 
 const ABOUT: &str = "
 ripgrep (rg) recursively searches your current directory for a regex pattern.
@@ -26,6 +27,9 @@ configuration file. The file can specify one shell argument per line. Lines
 starting with '#' are ignored. For more details, see the man page or the
 README.
 
+Tip: to disable all smart filtering and make ripgrep behave a bit more like
+classical grep, use 'rg -uuu'.
+
 Project home page: https://github.com/BurntSushi/ripgrep
 
 Use -h for short descriptions and --help for more details.";
@@ -543,7 +547,9 @@ pub fn all_args_and_flags() -> Vec<RGArg> {
     // flags are hidden and merely mentioned in the docs of the corresponding
     // "positive" flag.
     flag_after_context(&mut args);
+    flag_auto_hybrid_regex(&mut args);
     flag_before_context(&mut args);
+    flag_binary(&mut args);
     flag_block_buffered(&mut args);
     flag_byte_offset(&mut args);
     flag_case_sensitive(&mut args);
@@ -565,17 +571,20 @@ pub fn all_args_and_flags() -> Vec<RGArg> {
     flag_fixed_strings(&mut args);
     flag_follow(&mut args);
     flag_glob(&mut args);
+    flag_glob_case_insensitive(&mut args);
     flag_heading(&mut args);
     flag_hidden(&mut args);
     flag_iglob(&mut args);
     flag_ignore_case(&mut args);
     flag_ignore_file(&mut args);
+    flag_ignore_file_case_insensitive(&mut args);
     flag_invert_match(&mut args);
     flag_json(&mut args);
     flag_line_buffered(&mut args);
     flag_line_number(&mut args);
     flag_line_regexp(&mut args);
     flag_max_columns(&mut args);
+    flag_max_columns_preview(&mut args);
     flag_max_count(&mut args);
     flag_max_depth(&mut args);
     flag_max_filesize(&mut args);
@@ -584,6 +593,7 @@ pub fn all_args_and_flags() -> Vec<RGArg> {
     flag_multiline_dotall(&mut args);
     flag_no_config(&mut args);
     flag_no_ignore(&mut args);
+    flag_no_ignore_dot(&mut args);
     flag_no_ignore_global(&mut args);
     flag_no_ignore_messages(&mut args);
     flag_no_ignore_parent(&mut args);
@@ -597,6 +607,7 @@ pub fn all_args_and_flags() -> Vec<RGArg> {
     flag_path_separator(&mut args);
     flag_passthru(&mut args);
     flag_pcre2(&mut args);
+    flag_pcre2_version(&mut args);
     flag_pre(&mut args);
     flag_pre_glob(&mut args);
     flag_pretty(&mut args);
@@ -643,7 +654,7 @@ will be provided. Namely, the following is equivalent to the above:
     let arg = RGArg::positional("pattern", "PATTERN")
         .help(SHORT).long_help(LONG)
         .required_unless(&[
-            "file", "files", "regexp", "type-list",
+            "file", "files", "regexp", "type-list", "pcre2-version",
         ]);
     args.push(arg);
 }
@@ -674,6 +685,50 @@ This overrides the --context flag.
     args.push(arg);
 }
 
+fn flag_auto_hybrid_regex(args: &mut Vec<RGArg>) {
+    const SHORT: &str = "Dynamically use PCRE2 if necessary.";
+    const LONG: &str = long!("\
+When this flag is used, ripgrep will dynamically choose between supported regex
+engines depending on the features used in a pattern. When ripgrep chooses a
+regex engine, it applies that choice for every regex provided to ripgrep (e.g.,
+via multiple -e/--regexp or -f/--file flags).
+
+As an example of how this flag might behave, ripgrep will attempt to use
+its default finite automata based regex engine whenever the pattern can be
+successfully compiled with that regex engine. If PCRE2 is enabled and if the
+pattern given could not be compiled with the default regex engine, then PCRE2
+will be automatically used for searching. If PCRE2 isn't available, then this
+flag has no effect because there is only one regex engine to choose from.
+
+In the future, ripgrep may adjust its heuristics for how it decides which
+regex engine to use. In general, the heuristics will be limited to a static
+analysis of the patterns, and not to any specific runtime behavior observed
+while searching files.
+
+The primary downside of using this flag is that it may not always be obvious
+which regex engine ripgrep uses, and thus, the match semantics or performance
+profile of ripgrep may subtly and unexpectedly change. However, in many cases,
+all regex engines will agree on what constitutes a match and it can be nice
+to transparently support more advanced regex features like look-around and
+backreferences without explicitly needing to enable them.
+
+This flag can be disabled with --no-auto-hybrid-regex.
+");
+    let arg = RGArg::switch("auto-hybrid-regex")
+        .help(SHORT).long_help(LONG)
+        .overrides("no-auto-hybrid-regex")
+        .overrides("pcre2")
+        .overrides("no-pcre2");
+    args.push(arg);
+
+    let arg = RGArg::switch("no-auto-hybrid-regex")
+        .hidden()
+        .overrides("auto-hybrid-regex")
+        .overrides("pcre2")
+        .overrides("no-pcre2");
+    args.push(arg);
+}
+
 fn flag_before_context(args: &mut Vec<RGArg>) {
     const SHORT: &str = "Show NUM lines before each match.";
     const LONG: &str = long!("\
@@ -688,6 +743,55 @@ This overrides the --context flag.
     args.push(arg);
 }
 
+fn flag_binary(args: &mut Vec<RGArg>) {
+    const SHORT: &str = "Search binary files.";
+    const LONG: &str = long!("\
+Enabling this flag will cause ripgrep to search binary files. By default,
+ripgrep attempts to automatically skip binary files in order to improve the
+relevance of results and make the search faster.
+
+Binary files are heuristically detected based on whether they contain a NUL
+byte or not. By default (without this flag set), once a NUL byte is seen,
+ripgrep will stop searching the file. Usually, NUL bytes occur in the beginning
+of most binary files. If a NUL byte occurs after a match, then ripgrep will
+still stop searching the rest of the file, but a warning will be printed.
+
+In contrast, when this flag is provided, ripgrep will continue searching a file
+even if a NUL byte is found. In particular, if a NUL byte is found then ripgrep
+will continue searching until either a match is found or the end of the file is
+reached, whichever comes sooner. If a match is found, then ripgrep will stop
+and print a warning saying that the search stopped prematurely.
+
+If you want ripgrep to search a file without any special NUL byte handling at
+all (and potentially print binary data to stdout), then you should use the
+'-a/--text' flag.
+
+The '--binary' flag is a flag for controlling ripgrep's automatic filtering
+mechanism. As such, it does not need to be used when searching a file
+explicitly or when searching stdin. That is, it is only applicable when
+recursively searching a directory.
+
+Note that when the '-u/--unrestricted' flag is provided for a third time, then
+this flag is automatically enabled.
+
+This flag can be disabled with '--no-binary'. It overrides the '-a/--text'
+flag.
+");
+    let arg = RGArg::switch("binary")
+        .help(SHORT).long_help(LONG)
+        .overrides("no-binary")
+        .overrides("text")
+        .overrides("no-text");
+    args.push(arg);
+
+    let arg = RGArg::switch("no-binary")
+        .hidden()
+        .overrides("binary")
+        .overrides("text")
+        .overrides("no-text");
+    args.push(arg);
+}
+
 fn flag_block_buffered(args: &mut Vec<RGArg>) {
     const SHORT: &str = "Force block buffering.";
     const LONG: &str = long!("\
@@ -788,17 +892,17 @@ to one of eight choices: red, blue, green, cyan, magenta, yellow, white and
 black. Styles are limited to nobold, bold, nointense, intense, nounderline
 or underline.
 
-The format of the flag is `{type}:{attribute}:{value}`. `{type}` should be
-one of path, line, column or match. `{attribute}` can be fg, bg or style.
-`{value}` is either a color (for fg and bg) or a text style. A special format,
-`{type}:none`, will clear all color settings for `{type}`.
+The format of the flag is '{type}:{attribute}:{value}'. '{type}' should be
+one of path, line, column or match. '{attribute}' can be fg, bg or style.
+'{value}' is either a color (for fg and bg) or a text style. A special format,
+'{type}:none', will clear all color settings for '{type}'.
 
 For example, the following command will change the match color to magenta and
 the background color for line numbers to yellow:
 
     rg --colors 'match:fg:magenta' --colors 'line:bg:yellow' foo.
 
-Extended colors can be used for `{value}` when the terminal supports ANSI color
+Extended colors can be used for '{value}' when the terminal supports ANSI color
 sequences. These are specified as either 'x' (256-color) or 'x,x,x' (24-bit
 truecolor) where x is a number between 0 and 255 inclusive. x may be given as
 a normal decimal number or a hexadecimal number, which is prefixed by `0x`.
@@ -979,10 +1083,17 @@ fn flag_encoding(args: &mut Vec<RGArg>) {
     const LONG: &str = long!("\
 Specify the text encoding that ripgrep will use on all files searched. The
 default value is 'auto', which will cause ripgrep to do a best effort automatic
-detection of encoding on a per-file basis. Other supported values can be found
-in the list of labels here:
+detection of encoding on a per-file basis. Automatic detection in this case
+only applies to files that begin with a UTF-8 or UTF-16 byte-order mark (BOM).
+No other automatic detection is performed. One can also specify 'none' which
+will then completely disable BOM sniffing and always result in searching the
+raw bytes, including a BOM if it's present, regardless of its encoding.
+
+Other supported values can be found in the list of labels here:
 https://encoding.spec.whatwg.org/#concept-encoding-get
 
+For more details on encoding and how ripgrep deals with it, see GUIDE.md.
+
 This flag can be disabled with --no-encoding.
 ");
     let arg = RGArg::flag("encoding", "ENCODING").short("E")
@@ -1016,7 +1127,7 @@ fn flag_files(args: &mut Vec<RGArg>) {
     const SHORT: &str = "Print each file that would be searched.";
     const LONG: &str = long!("\
 Print each file that would be searched without actually performing the search.
-This is useful to determine whether a particular file is being search or not.
+This is useful to determine whether a particular file is being searched or not.
 ");
     let arg = RGArg::switch("files")
         .help(SHORT).long_help(LONG)
@@ -1108,6 +1219,25 @@ it.
     args.push(arg);
 }
 
+fn flag_glob_case_insensitive(args: &mut Vec<RGArg>) {
+    const SHORT: &str = "Process all glob patterns case insensitively.";
+    const LONG: &str = long!("\
+Process glob patterns given with the -g/--glob flag case insensitively. This
+effectively treats --glob as --iglob.
+
+This flag can be disabled with the --no-glob-case-insensitive flag.
+");
+    let arg = RGArg::switch("glob-case-insensitive")
+        .help(SHORT).long_help(LONG)
+        .overrides("no-glob-case-insensitive");
+    args.push(arg);
+
+    let arg = RGArg::switch("no-glob-case-insensitive")
+        .hidden()
+        .overrides("glob-case-insensitive");
+    args.push(arg);
+}
+
 fn flag_heading(args: &mut Vec<RGArg>) {
     const SHORT: &str = "Print matches grouped by each file.";
     const LONG: &str = long!("\
@@ -1208,6 +1338,26 @@ directly on the command line, then used -g instead.
     args.push(arg);
 }
 
+fn flag_ignore_file_case_insensitive(args: &mut Vec<RGArg>) {
+    const SHORT: &str = "Process ignore files case insensitively.";
+    const LONG: &str = long!("\
+Process ignore files (.gitignore, .ignore, etc.) case insensitively. Note that
+this comes with a performance penalty and is most useful on case insensitive
+file systems (such as Windows).
+
+This flag can be disabled with the --no-ignore-file-case-insensitive flag.
+");
+    let arg = RGArg::switch("ignore-file-case-insensitive")
+        .help(SHORT).long_help(LONG)
+        .overrides("no-ignore-file-case-insensitive");
+    args.push(arg);
+
+    let arg = RGArg::switch("no-ignore-file-case-insensitive")
+        .hidden()
+        .overrides("ignore-file-case-insensitive");
+    args.push(arg);
+}
+
 fn flag_invert_match(args: &mut Vec<RGArg>) {
     const SHORT: &str = "Invert matching.";
     const LONG: &str = long!("\
@@ -1360,6 +1510,30 @@ When this flag is omitted or is set to 0, then it has no effect.
     args.push(arg);
 }
 
+fn flag_max_columns_preview(args: &mut Vec<RGArg>) {
+    const SHORT: &str = "Print a preview for lines exceeding the limit.";
+    const LONG: &str = long!("\
+When the '--max-columns' flag is used, ripgrep will by default completely
+replace any line that is too long with a message indicating that a matching
+line was removed. When this flag is combined with '--max-columns', a preview
+of the line (corresponding to the limit size) is shown instead, where the part
+of the line exceeding the limit is not shown.
+
+If the '--max-columns' flag is not set, then this has no effect.
+
+This flag can be disabled with '--no-max-columns-preview'.
+");
+    let arg = RGArg::switch("max-columns-preview")
+        .help(SHORT).long_help(LONG)
+        .overrides("no-max-columns-preview");
+    args.push(arg);
+
+    let arg = RGArg::switch("no-max-columns-preview")
+        .hidden()
+        .overrides("max-columns-preview");
+    args.push(arg);
+}
+
 fn flag_max_count(args: &mut Vec<RGArg>) {
     const SHORT: &str = "Limit the number of matches.";
     const LONG: &str = long!("\
@@ -1535,7 +1709,7 @@ fn flag_no_ignore(args: &mut Vec<RGArg>) {
     const SHORT: &str = "Don't respect ignore files.";
     const LONG: &str = long!("\
 Don't respect ignore files (.gitignore, .ignore, etc.). This implies
---no-ignore-parent and --no-ignore-vcs.
+--no-ignore-parent, --no-ignore-dot and --no-ignore-vcs.
 
 This flag can be disabled with the --ignore flag.
 ");
@@ -1550,6 +1724,24 @@ This flag can be disabled with the --ignore flag.
     args.push(arg);
 }
 
+fn flag_no_ignore_dot(args: &mut Vec<RGArg>) {
+    const SHORT: &str = "Don't respect .ignore files.";
+    const LONG: &str = long!("\
+Don't respect .ignore files.
+
+This flag can be disabled with the --ignore-dot flag.
+");
+    let arg = RGArg::switch("no-ignore-dot")
+        .help(SHORT).long_help(LONG)
+        .overrides("ignore-dot");
+    args.push(arg);
+
+    let arg = RGArg::switch("ignore-dot")
+        .hidden()
+        .overrides("no-ignore-dot");
+    args.push(arg);
+}
+
 fn flag_no_ignore_global(args: &mut Vec<RGArg>) {
     const SHORT: &str = "Don't respect global ignore files.";
     const LONG: &str = long!("\
@@ -1803,7 +1995,12 @@ or backreferences.
 
 Note that PCRE2 is an optional ripgrep feature. If PCRE2 wasn't included in
 your build of ripgrep, then using this flag will result in ripgrep printing
-an error message and exiting.
+an error message and exiting. PCRE2 may also have worse user experience in
+some cases, since it has fewer introspection APIs than ripgrep's default regex
+engine. For example, if you use a '\n' in a PCRE2 regex without the
+'-U/--multiline' flag, then ripgrep will silently fail to match anything
+instead of reporting an error immediately (like it does with the default
+regex engine).
 
 Related flags: --no-pcre2-unicode
 
@@ -1811,12 +2008,28 @@ This flag can be disabled with --no-pcre2.
 ");
     let arg = RGArg::switch("pcre2").short("P")
         .help(SHORT).long_help(LONG)
-        .overrides("no-pcre2");
+        .overrides("no-pcre2")
+        .overrides("auto-hybrid-regex")
+        .overrides("no-auto-hybrid-regex");
     args.push(arg);
 
     let arg = RGArg::switch("no-pcre2")
         .hidden()
-        .overrides("pcre2");
+        .overrides("pcre2")
+        .overrides("auto-hybrid-regex")
+        .overrides("no-auto-hybrid-regex");
+    args.push(arg);
+}
+
+fn flag_pcre2_version(args: &mut Vec<RGArg>) {
+    const SHORT: &str = "Print the version of PCRE2 that ripgrep uses.";
+    const LONG: &str = long!("\
+When this flag is present, ripgrep will print the version of PCRE2 in use,
+along with other information, and then exit. If PCRE2 is not available, then
+ripgrep will print an error message and exit with an error code.
+");
+    let arg = RGArg::switch("pcre2-version")
+        .help(SHORT).long_help(LONG);
     args.push(arg);
 }
 
@@ -1826,12 +2039,13 @@ fn flag_pre(args: &mut Vec<RGArg>) {
 For each input FILE, search the standard output of COMMAND FILE rather than the
 contents of FILE. This option expects the COMMAND program to either be an
 absolute path or to be available in your PATH. Either an empty string COMMAND
-or the `--no-pre` flag will disable this behavior.
+or the '--no-pre' flag will disable this behavior.
 
     WARNING: When this flag is set, ripgrep will unconditionally spawn a
     process for every file that is searched. Therefore, this can incur an
     unnecessarily large performance penalty if you don't otherwise need the
-    flexibility offered by this flag.
+    flexibility offered by this flag. One possible mitigation to this is to use
+    the '--pre-glob' flag to limit which files a preprocessor is run with.
 
 A preprocessor is not run when ripgrep is searching stdin.
 
@@ -1982,7 +2196,10 @@ Replace every match with the text given when printing results. Neither this
 flag nor any other ripgrep flag will modify your files.
 
 Capture group indices (e.g., $5) and names (e.g., $foo) are supported in the
-replacement string.
+replacement string. In shells such as Bash and zsh, you should wrap the
+pattern in single quotes instead of double quotes. Otherwise, capture group
+indices will be replaced by expanded shell variables which will most likely
+be empty.
 
 Note that the replacement by default replaces each match, and NOT the entire
 line. To replace the entire line, you should match the entire line.
@@ -1998,9 +2215,9 @@ This flag can be used with the -o/--only-matching flag.
 fn flag_search_zip(args: &mut Vec<RGArg>) {
     const SHORT: &str = "Search in compressed files.";
     const LONG: &str = long!("\
-Search in compressed files. Currently gz, bz2, xz, lzma and lz4 files are
-supported. This option expects the decompression binaries to be available in
-your PATH.
+Search in compressed files. Currently gzip, bzip2, xz, LZ4, LZMA, Brotli and
+Zstd files are supported. This option expects the decompression binaries to be
+available in your PATH.
 
 This flag can be disabled with --no-search-zip.
 ");
@@ -2067,7 +2284,7 @@ for this flag are:
     path        Sort by file path.
     modified    Sort by the last modified time on a file.
     accessed    Sort by the last accessed time on a file.
-    created     Sort by the cretion time on a file.
+    created     Sort by the creation time on a file.
     none        Do not sort results.
 
 If the sorting criteria isn't available on your system (for example, creation
@@ -2100,7 +2317,7 @@ for this flag are:
     path        Sort by file path.
     modified    Sort by the last modified time on a file.
     accessed    Sort by the last accessed time on a file.
-    created     Sort by the cretion time on a file.
+    created     Sort by the creation time on a file.
     none        Do not sort results.
 
 If the sorting criteria isn't available on your system (for example, creation
@@ -2160,20 +2377,23 @@ escape codes to be printed that alter the behavior of your terminal.
 When binary file detection is enabled it is imperfect. In general, it uses
 a simple heuristic. If a NUL byte is seen during search, then the file is
 considered binary and search stops (unless this flag is present).
+Alternatively, if the '--binary' flag is used, then ripgrep will only quit
+when it sees a NUL byte after it sees a match (or searches the entire file).
 
-Note that when the `-u/--unrestricted` flag is provided for a third time, then
-this flag is automatically enabled.
-
-This flag can be disabled with --no-text.
+This flag can be disabled with '--no-text'. It overrides the '--binary' flag.
 ");
     let arg = RGArg::switch("text").short("a")
         .help(SHORT).long_help(LONG)
-        .overrides("no-text");
+        .overrides("no-text")
+        .overrides("binary")
+        .overrides("no-binary");
     args.push(arg);
 
     let arg = RGArg::switch("no-text")
         .hidden()
-        .overrides("text");
+        .overrides("text")
+        .overrides("binary")
+        .overrides("no-binary");
     args.push(arg);
 }
 
@@ -2302,8 +2522,7 @@ Reduce the level of \"smart\" searching. A single -u won't respect .gitignore
 (etc.) files. Two -u flags will additionally search hidden files and
 directories. Three -u flags will additionally search binary files.
 
--uu is roughly equivalent to grep -r and -uuu is roughly equivalent to grep -a
--r.
+'rg -uuu' is roughly equivalent to 'grep -r'.
 ");
     let arg = RGArg::switch("unrestricted").short("u")
         .help(SHORT).long_help(LONG)
@@ -2345,7 +2564,7 @@ ripgrep is explicitly instructed to search one file or stdin.
 
 This flag overrides --with-filename.
 ");
-    let arg = RGArg::switch("no-filename")
+    let arg = RGArg::switch("no-filename").short("I")
         .help(NO_SHORT).long_help(NO_LONG)
         .overrides("with-filename");
     args.push(arg);
