@@ -1,9 +1,10 @@
 use std::cmp;
 use std::env;
-use std::ffi::OsStr;
+use std::ffi::{OsStr, OsString};
 use std::fs;
-use std::io;
+use std::io::{self, Write};
 use std::path::{Path, PathBuf};
+use std::process;
 use std::sync::Arc;
 use std::time::SystemTime;
 
@@ -34,20 +35,22 @@ use ignore::types::{FileTypeDef, Types, TypesBuilder};
 use ignore::{Walk, WalkBuilder, WalkParallel};
 use log;
 use num_cpus;
-use path_printer::{PathPrinter, PathPrinterBuilder};
 use regex;
 use termcolor::{
     WriteColor,
     BufferWriter, ColorChoice,
 };
 
-use app;
-use config;
-use logger::Logger;
-use messages::{set_messages, set_ignore_messages};
-use search::{PatternMatcher, Printer, SearchWorker, SearchWorkerBuilder};
-use subject::SubjectBuilder;
-use Result;
+use crate::app;
+use crate::config;
+use crate::logger::Logger;
+use crate::messages::{set_messages, set_ignore_messages};
+use crate::path_printer::{PathPrinter, PathPrinterBuilder};
+use crate::search::{
+    PatternMatcher, Printer, SearchWorker, SearchWorkerBuilder,
+};
+use crate::subject::SubjectBuilder;
+use crate::Result;
 
 /// The command that ripgrep should execute based on the command line
 /// configuration.
@@ -70,6 +73,8 @@ pub enum Command {
     /// List all file type definitions configured, including the default file
     /// types and any additional file types added to the command line.
     Types,
+    /// Print the version of PCRE2 in use.
+    PCRE2Version,
 }
 
 impl Command {
@@ -79,7 +84,11 @@ impl Command {
 
         match *self {
             Search | SearchParallel => true,
-            SearchNever | Files | FilesParallel | Types => false,
+            | SearchNever
+            | Files
+            | FilesParallel
+            | Types
+            | PCRE2Version => false,
         }
     }
 }
@@ -128,7 +137,7 @@ impl Args {
         // trying to parse config files. If a config file exists and has
         // arguments, then we re-parse argv, otherwise we just use the matches
         // we have here.
-        let early_matches = ArgMatches::new(app::app().get_matches());
+        let early_matches = ArgMatches::new(clap_matches(env::args_os())?);
         set_messages(!early_matches.is_present("no-messages"));
         set_ignore_messages(!early_matches.is_present("no-ignore-messages"));
 
@@ -143,7 +152,7 @@ impl Args {
             log::set_max_level(log::LevelFilter::Warn);
         }
 
-        let matches = early_matches.reconfigure();
+        let matches = early_matches.reconfigure()?;
         // The logging level may have changed if we brought in additional
         // arguments from a configuration file, so recheck it and set the log
         // level as appropriate.
@@ -232,7 +241,9 @@ impl Args {
         let threads = self.matches().threads()?;
         let one_thread = is_one_search || threads == 1;
 
-        Ok(if self.matches().is_present("type-list") {
+        Ok(if self.matches().is_present("pcre2-version") {
+            Command::PCRE2Version
+        } else if self.matches().is_present("type-list") {
             Command::Types
         } else if self.matches().is_present("files") {
             if one_thread {
@@ -265,6 +276,11 @@ impl Args {
         Ok(builder.build(wtr))
     }
 
+    /// Returns true if and only if ripgrep should be "quiet."
+    pub fn quiet(&self) -> bool {
+        self.matches().is_present("quiet")
+    }
+
     /// Returns true if and only if the search should quit after finding the
     /// first match.
     pub fn quit_after_match(&self) -> Result<bool> {
@@ -278,15 +294,18 @@ impl Args {
         &self,
         wtr: W,
     ) -> Result<SearchWorker<W>> {
+        let matches = self.matches();
         let matcher = self.matcher().clone();
         let printer = self.printer(wtr)?;
-        let searcher = self.matches().searcher(self.paths())?;
+        let searcher = matches.searcher(self.paths())?;
         let mut builder = SearchWorkerBuilder::new();
         builder
-            .json_stats(self.matches().is_present("json"))
-            .preprocessor(self.matches().preprocessor())
-            .preprocessor_globs(self.matches().preprocessor_globs()?)
-            .search_zip(self.matches().is_present("search-zip"));
+            .json_stats(matches.is_present("json"))
+            .preprocessor(matches.preprocessor())
+            .preprocessor_globs(matches.preprocessor_globs()?)
+            .search_zip(matches.is_present("search-zip"))
+            .binary_detection_implicit(matches.binary_detection_implicit())
+            .binary_detection_explicit(matches.binary_detection_explicit());
         Ok(builder.build(matcher, searcher, printer))
     }
 
@@ -475,6 +494,37 @@ impl SortByKind {
     }
 }
 
+/// Encoding mode the searcher will use.
+#[derive(Clone, Debug)]
+enum EncodingMode {
+    /// Use an explicit encoding forcefully, but let BOM sniffing override it.
+    Some(Encoding),
+    /// Use only BOM sniffing to auto-detect an encoding.
+    Auto,
+    /// Use no explicit encoding and disable all BOM sniffing. This will
+    /// always result in searching the raw bytes, regardless of their
+    /// true encoding.
+    Disabled,
+}
+
+impl EncodingMode {
+    /// Checks if an explicit encoding has been set. Returns false for
+    /// automatic BOM sniffing and no sniffing.
+    ///
+    /// This is only used to determine whether PCRE2 needs to have its own
+    /// UTF-8 checking enabled. If we have an explicit encoding set, then
+    /// we're always guaranteed to get UTF-8, so we can disable PCRE2's check.
+    /// Otherwise, we have no such guarantee, and must enable PCRE2' UTF-8
+    /// check.
+    #[cfg(feature = "pcre2")]
+    fn has_explicit_encoding(&self) -> bool {
+        match self {
+            EncodingMode::Some(_) => true,
+            _ => false
+        }
+    }
+}
+
 impl ArgMatches {
     /// Create an ArgMatches from clap's parse result.
     fn new(clap_matches: clap::ArgMatches<'static>) -> ArgMatches {
@@ -488,25 +538,27 @@ impl ArgMatches {
     ///
     /// If there are no additional arguments from the environment (e.g., a
     /// config file), then the given matches are returned as is.
-    fn reconfigure(self) -> ArgMatches {
+    fn reconfigure(self) -> Result<ArgMatches> {
         // If the end user says no config, then respect it.
         if self.is_present("no-config") {
-            debug!("not reading config files because --no-config is present");
-            return self;
+            log::debug!(
+                "not reading config files because --no-config is present"
+            );
+            return Ok(self);
         }
         // If the user wants ripgrep to use a config file, then parse args
         // from that first.
         let mut args = config::args();
         if args.is_empty() {
-            return self;
+            return Ok(self);
         }
         let mut cliargs = env::args_os();
         if let Some(bin) = cliargs.next() {
             args.insert(0, bin);
         }
         args.extend(cliargs);
-        debug!("final argv: {:?}", args);
-        ArgMatches::new(app::app().get_matches_from(args))
+        log::debug!("final argv: {:?}", args);
+        Ok(ArgMatches(clap_matches(args)?))
     }
 
     /// Convert the result of parsing CLI arguments into ripgrep's higher level
@@ -547,6 +599,25 @@ impl ArgMatches {
         if self.is_present("pcre2") {
             let matcher = self.matcher_pcre2(patterns)?;
             Ok(PatternMatcher::PCRE2(matcher))
+        } else if self.is_present("auto-hybrid-regex") {
+            let rust_err = match self.matcher_rust(patterns) {
+                Ok(matcher) => return Ok(PatternMatcher::RustRegex(matcher)),
+                Err(err) => err,
+            };
+            log::debug!(
+                "error building Rust regex in hybrid mode:\n{}", rust_err,
+            );
+            let pcre_err = match self.matcher_pcre2(patterns) {
+                Ok(matcher) => return Ok(PatternMatcher::PCRE2(matcher)),
+                Err(err) => err,
+            };
+            Err(From::from(format!(
+                "regex could not be compiled with either the default regex \
+                 engine or with PCRE2.\n\n\
+                 default regex engine error:\n{}\n{}\n{}\n\n\
+                 PCRE2 regex engine error:\n{}",
+                 "~".repeat(79), rust_err, "~".repeat(79), pcre_err,
+            )))
         } else {
             let matcher = match self.matcher_rust(patterns) {
                 Ok(matcher) => matcher,
@@ -615,7 +686,16 @@ impl ArgMatches {
         if let Some(limit) = self.dfa_size_limit()? {
             builder.dfa_size_limit(limit);
         }
-        Ok(builder.build(&patterns.join("|"))?)
+        let res =
+            if self.is_present("fixed-strings") {
+                builder.build_literals(patterns)
+            } else {
+                builder.build(&patterns.join("|"))
+            };
+        match res {
+            Ok(m) => Ok(m),
+            Err(err) => Err(From::from(suggest_multiline(err.to_string()))),
+        }
     }
 
     /// Build a matcher using PCRE2.
@@ -632,12 +712,17 @@ impl ArgMatches {
             .word(self.is_present("word-regexp"));
         // For whatever reason, the JIT craps out during regex compilation with
         // a "no more memory" error on 32 bit systems. So don't use it there.
-        if !cfg!(target_pointer_width = "32") {
-            builder.jit(true);
+        if cfg!(target_pointer_width = "64") {
+            builder
+                .jit_if_available(true)
+                // The PCRE2 docs say that 32KB is the default, and that 1MB
+                // should be big enough for anything. But let's crank it to
+                // 10MB.
+                .max_jit_stack_size(Some(10 * (1<<20)));
         }
         if self.pcre2_unicode() {
             builder.utf(true).ucp(true);
-            if self.encoding()?.is_some() {
+            if self.encoding()?.has_explicit_encoding() {
                 // SAFETY: If an encoding was specified, then we're guaranteed
                 // to get valid UTF-8, so we can disable PCRE2's UTF checking.
                 // (Feeding invalid UTF-8 to PCRE2 is undefined behavior.)
@@ -693,6 +778,7 @@ impl ArgMatches {
             .per_match(self.is_present("vimgrep"))
             .replacement(self.replacement())
             .max_columns(self.max_columns()?)
+            .max_columns_preview(self.max_columns_preview())
             .max_matches(self.max_count()?)
             .column(self.column())
             .byte_offset(self.is_present("byte-offset"))
@@ -752,9 +838,16 @@ impl ArgMatches {
             .before_context(ctx_before)
             .after_context(ctx_after)
             .passthru(self.is_present("passthru"))
-            .memory_map(self.mmap_choice(paths))
-            .binary_detection(self.binary_detection())
-            .encoding(self.encoding()?);
+            .memory_map(self.mmap_choice(paths));
+        match self.encoding()? {
+            EncodingMode::Some(enc) => {
+                builder.encoding(Some(enc));
+            }
+            EncodingMode::Auto => {} // default for the searcher
+            EncodingMode::Disabled => {
+                builder.bom_sniffing(false);
+            }
+        }
         Ok(builder.build())
     }
 
@@ -779,18 +872,16 @@ impl ArgMatches {
             .max_filesize(self.max_file_size()?)
             .threads(self.threads()?)
             .same_file_system(self.is_present("one-file-system"))
-            .skip_stdout(true)
+            .skip_stdout(!self.is_present("files"))
             .overrides(self.overrides()?)
             .types(self.types()?)
             .hidden(!self.hidden())
             .parents(!self.no_ignore_parent())
-            .ignore(!self.no_ignore())
-            .git_global(
-                !self.no_ignore()
-                && !self.no_ignore_vcs()
-                && !self.no_ignore_global())
-            .git_ignore(!self.no_ignore() && !self.no_ignore_vcs())
-            .git_exclude(!self.no_ignore() && !self.no_ignore_vcs());
+            .ignore(!self.no_ignore_dot())
+            .git_global(!self.no_ignore_vcs() && !self.no_ignore_global())
+            .git_ignore(!self.no_ignore_vcs())
+            .git_exclude(!self.no_ignore_vcs())
+            .ignore_case_insensitive(self.ignore_file_case_insensitive());
         if !self.no_ignore() {
             builder.add_custom_ignore_filename(".rgignore");
         }
@@ -806,16 +897,39 @@ impl ArgMatches {
 ///
 /// Methods are sorted alphabetically.
 impl ArgMatches {
-    /// Returns the form of binary detection to perform.
-    fn binary_detection(&self) -> BinaryDetection {
+    /// Returns the form of binary detection to perform on files that are
+    /// implicitly searched via recursive directory traversal.
+    fn binary_detection_implicit(&self) -> BinaryDetection {
+        let none =
+            self.is_present("text")
+            || self.is_present("null-data");
+        let convert =
+            self.is_present("binary")
+            || self.unrestricted_count() >= 3;
+        if none {
+            BinaryDetection::none()
+        } else if convert {
+            BinaryDetection::convert(b'\x00')
+        } else {
+            BinaryDetection::quit(b'\x00')
+        }
+    }
+
+    /// Returns the form of binary detection to perform on files that are
+    /// explicitly searched via the user invoking ripgrep on a particular
+    /// file or files or stdin.
+    ///
+    /// In general, this should never be BinaryDetection::quit, since that acts
+    /// as a filter (but quitting immediately once a NUL byte is seen), and we
+    /// should never filter out files that the user wants to explicitly search.
+    fn binary_detection_explicit(&self) -> BinaryDetection {
         let none =
             self.is_present("text")
-            || self.unrestricted_count() >= 3
             || self.is_present("null-data");
         if none {
             BinaryDetection::none()
         } else {
-            BinaryDetection::quit(b'\x00')
+            BinaryDetection::convert(b'\x00')
         }
     }
 
@@ -941,24 +1055,30 @@ impl ArgMatches {
         u64_to_usize("dfa-size-limit", r)
     }
 
-    /// Returns the type of encoding to use.
+    /// Returns the encoding mode to use.
     ///
-    /// This only returns an encoding if one is explicitly specified. When no
-    /// encoding is present, the Searcher will still do BOM sniffing for UTF-16
-    /// and transcode seamlessly.
-    fn encoding(&self) -> Result<Option<Encoding>> {
+    /// This only returns an encoding if one is explicitly specified. Otherwise
+    /// if set to automatic, the Searcher will do BOM sniffing for UTF-16
+    /// and transcode seamlessly. If disabled, no BOM sniffing nor transcoding
+    /// will occur.
+    fn encoding(&self) -> Result<EncodingMode> {
         if self.is_present("no-encoding") {
-            return Ok(None);
+            return Ok(EncodingMode::Auto);
         }
+
         let label = match self.value_of_lossy("encoding") {
             None if self.pcre2_unicode() => "utf-8".to_string(),
-            None => return Ok(None),
+            None => return Ok(EncodingMode::Auto),
             Some(label) => label,
         };
+
         if label == "auto" {
-            return Ok(None);
+            return Ok(EncodingMode::Auto);
+        } else if label == "none" {
+            return Ok(EncodingMode::Disabled);
         }
-        Ok(Some(Encoding::new(&label)?))
+
+        Ok(EncodingMode::Some(Encoding::new(&label)?))
     }
 
     /// Return the file separator to use based on the CLI configuration.
@@ -996,6 +1116,11 @@ impl ArgMatches {
         self.is_present("hidden") || self.unrestricted_count() >= 2
     }
 
+    /// Returns true if ignore files should be processed case insensitively.
+    fn ignore_file_case_insensitive(&self) -> bool {
+        self.is_present("ignore-file-case-insensitive")
+    }
+
     /// Return all of the ignore file paths given on the command line.
     fn ignore_paths(&self) -> Vec<PathBuf> {
         let paths = match self.values_of_os("ignore-file") {
@@ -1050,6 +1175,12 @@ impl ArgMatches {
         Ok(self.usize_of_nonzero("max-columns")?.map(|n| n as u64))
     }
 
+    /// Returns true if and only if a preview should be shown for lines that
+    /// exceed the maximum column limit.
+    fn max_columns_preview(&self) -> bool {
+        self.is_present("max-columns-preview")
+    }
+
     /// The maximum number of matches permitted.
     fn max_count(&self) -> Result<Option<u64>> {
         Ok(self.usize_of("max-count")?.map(|n| n as u64))
@@ -1090,6 +1221,11 @@ impl ArgMatches {
         self.is_present("no-ignore") || self.unrestricted_count() >= 1
     }
 
+    /// Returns true if .ignore files should be ignored.
+    fn no_ignore_dot(&self) -> bool {
+        self.is_present("no-ignore-dot") || self.no_ignore()
+    }
+
     /// Returns true if global ignore files should be ignored.
     fn no_ignore_global(&self) -> bool {
         self.is_present("no-ignore-global") || self.no_ignore()
@@ -1132,11 +1268,15 @@ impl ArgMatches {
     /// Builds the set of glob overrides from the command line flags.
     fn overrides(&self) -> Result<Override> {
         let mut builder = OverrideBuilder::new(env::current_dir()?);
+        // Make all globs case insensitive with --glob-case-insensitive.
+        if self.is_present("glob-case-insensitive") {
+            builder.case_insensitive(true).unwrap();
+        }
         for glob in self.values_of_lossy_vec("glob") {
             builder.add(&glob)?;
         }
         // This only enables case insensitivity for subsequent globs.
-        builder.case_insensitive(true)?;
+        builder.case_insensitive(true).unwrap();
         for glob in self.values_of_lossy_vec("iglob") {
             builder.add(&glob)?;
         }
@@ -1174,7 +1314,8 @@ impl ArgMatches {
             !cli::is_readable_stdin()
             || (self.is_present("file") && file_is_stdin)
             || self.is_present("files")
-            || self.is_present("type-list");
+            || self.is_present("type-list")
+            || self.is_present("pcre2-version");
         if search_cwd {
             Path::new("./").to_path_buf()
         } else {
@@ -1250,9 +1391,15 @@ impl ArgMatches {
         if let Some(paths) = self.values_of_os("file") {
             for path in paths {
                 if path == "-" {
-                    pats.extend(cli::patterns_from_stdin()?);
+                    pats.extend(cli::patterns_from_stdin()?
+                        .into_iter()
+                        .map(|p| self.pattern_from_string(p))
+                    );
                 } else {
-                    pats.extend(cli::patterns_from_path(path)?);
+                    pats.extend(cli::patterns_from_path(path)?
+                        .into_iter()
+                        .map(|p| self.pattern_from_string(p))
+                    );
                 }
             }
         }
@@ -1281,13 +1428,17 @@ impl ArgMatches {
     /// Converts a &str pattern to a String pattern. The pattern is escaped
     /// if -F/--fixed-strings is set.
     fn pattern_from_str(&self, pat: &str) -> String {
-        let litpat = self.pattern_literal(pat.to_string());
-        let s = self.pattern_line(litpat);
+        self.pattern_from_string(pat.to_string())
+    }
 
-        if s.is_empty() {
+    /// Applies additional processing on the given pattern if necessary
+    /// (such as escaping meta characters or turning it into a line regex).
+    fn pattern_from_string(&self, pat: String) -> String {
+        let pat = self.pattern_line(self.pattern_literal(pat));
+        if pat.is_empty() {
             self.pattern_empty()
         } else {
-            s
+            pat
         }
     }
 
@@ -1443,10 +1594,11 @@ impl ArgMatches {
         if self.is_present("no-filename") {
             false
         } else {
+            let path_stdin = Path::new("-");
             self.is_present("with-filename")
             || self.is_present("vimgrep")
             || paths.len() > 1
-            || paths.get(0).map_or(false, |p| p.is_dir())
+            || paths.get(0).map_or(false, |p| p != path_stdin && p.is_dir())
         }
     }
 }
@@ -1546,6 +1698,17 @@ and look-around.", msg)
     }
 }
 
+fn suggest_multiline(msg: String) -> String {
+    if msg.contains("the literal") && msg.contains("not allowed") {
+        format!("{}
+
+Consider enabling multiline mode with the --multiline flag (or -U for short).
+When multiline mode is enabled, new line characters can be matched.", msg)
+    } else {
+        msg
+    }
+}
+
 /// Convert the result of parsing a human readable file size to a `usize`,
 /// failing if the type does not fit.
 fn u64_to_usize(
@@ -1592,3 +1755,32 @@ where G: Fn(&fs::Metadata) -> io::Result<SystemTime>
         t1.cmp(&t2)
     }
 }
+
+/// Returns a clap matches object if the given arguments parse successfully.
+///
+/// Otherwise, if an error occurred, then it is returned unless the error
+/// corresponds to a `--help` or `--version` request. In which case, the
+/// corresponding output is printed and the current process is exited
+/// successfully.
+fn clap_matches<I, T>(
+    args: I,
+) -> Result<clap::ArgMatches<'static>>
+where I: IntoIterator<Item=T>,
+      T: Into<OsString> + Clone
+{
+    let err = match app::app().get_matches_from_safe(args) {
+        Ok(matches) => return Ok(matches),
+        Err(err) => err,
+    };
+    if err.use_stderr() {
+        return Err(err.into());
+    }
+    // Explicitly ignore any error returned by write!. The most likely error
+    // at this point is a broken pipe error, in which case, we want to ignore
+    // it and exit quietly.
+    //
+    // (This is the point of this helper function. clap's functionality for
+    // doing this will panic on a broken pipe error.)
+    let _ = write!(io::stdout(), "{}", err);
+    process::exit(0);
+}
