@@ -1,5 +1,5 @@
-use hay::SHERLOCK;
-use util::{Dir, TestCommand, cmd_exists, sort_lines};
+use crate::hay::SHERLOCK;
+use crate::util::{Dir, TestCommand, cmd_exists, sort_lines};
 
 // This file contains "miscellaneous" tests that were either written before
 // features were tracked more explicitly, or were simply written without
@@ -341,6 +341,14 @@ rgtest!(glob_case_sensitive, |dir: Dir, mut cmd: TestCommand| {
     eqnice!("file2.html:Sherlock\n", cmd.stdout());
 });
 
+rgtest!(glob_always_case_insensitive, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("sherlock", SHERLOCK);
+    dir.create("file.HTML", "Sherlock");
+    cmd.args(&["--glob-case-insensitive", "--glob", "*.html", "Sherlock"]);
+
+    eqnice!("file.HTML:Sherlock\n", cmd.stdout());
+});
+
 rgtest!(byte_offset_only_matching, |dir: Dir, mut cmd: TestCommand| {
     dir.create("sherlock", SHERLOCK);
     cmd.arg("-b").arg("-o").arg("Sherlock");
@@ -752,12 +760,11 @@ rgtest!(unrestricted2, |dir: Dir, mut cmd: TestCommand| {
 
 rgtest!(unrestricted3, |dir: Dir, mut cmd: TestCommand| {
     dir.create("sherlock", SHERLOCK);
-    dir.create("file", "foo\x00bar\nfoo\x00baz\n");
+    dir.create("hay", "foo\x00bar\nfoo\x00baz\n");
     cmd.arg("-uuu").arg("foo");
 
     let expected = "\
-file:foo\x00bar
-file:foo\x00baz
+Binary file hay matches (found \"\\u{0}\" byte around offset 3)
 ";
     eqnice!(expected, cmd.stdout());
 });
@@ -909,6 +916,36 @@ be, to a very large extent, the result of luck. Sherlock Holmes
     eqnice!(expected, cmd.stdout());
 });
 
+rgtest!(compressed_brotli, |dir: Dir, mut cmd: TestCommand| {
+    if !cmd_exists("brotli") {
+        return;
+    }
+
+    dir.create_bytes("sherlock.br", include_bytes!("./data/sherlock.br"));
+    cmd.arg("-z").arg("Sherlock").arg("sherlock.br");
+
+    let expected = "\
+For the Doctor Watsons of this world, as opposed to the Sherlock
+be, to a very large extent, the result of luck. Sherlock Holmes
+";
+    eqnice!(expected, cmd.stdout());
+});
+
+rgtest!(compressed_zstd, |dir: Dir, mut cmd: TestCommand| {
+    if !cmd_exists("zstd") {
+        return;
+    }
+
+    dir.create_bytes("sherlock.zst", include_bytes!("./data/sherlock.zst"));
+    cmd.arg("-z").arg("Sherlock").arg("sherlock.zst");
+
+    let expected = "\
+For the Doctor Watsons of this world, as opposed to the Sherlock
+be, to a very large extent, the result of luck. Sherlock Holmes
+";
+    eqnice!(expected, cmd.stdout());
+});
+
 rgtest!(compressed_failing_gzip, |dir: Dir, mut cmd: TestCommand| {
     if !cmd_exists("gzip") {
         return;
@@ -920,10 +957,35 @@ rgtest!(compressed_failing_gzip, |dir: Dir, mut cmd: TestCommand| {
     cmd.assert_non_empty_stderr();
 });
 
-rgtest!(binary_nosearch, |dir: Dir, mut cmd: TestCommand| {
+rgtest!(binary_convert, |dir: Dir, mut cmd: TestCommand| {
     dir.create("file", "foo\x00bar\nfoo\x00baz\n");
-    cmd.arg("foo").arg("file");
+    cmd.arg("--no-mmap").arg("foo").arg("file");
 
+    let expected = "\
+Binary file matches (found \"\\u{0}\" byte around offset 3)
+";
+    eqnice!(expected, cmd.stdout());
+});
+
+rgtest!(binary_convert_mmap, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("file", "foo\x00bar\nfoo\x00baz\n");
+    cmd.arg("--mmap").arg("foo").arg("file");
+
+    let expected = "\
+Binary file matches (found \"\\u{0}\" byte around offset 3)
+";
+    eqnice!(expected, cmd.stdout());
+});
+
+rgtest!(binary_quit, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("file", "foo\x00bar\nfoo\x00baz\n");
+    cmd.arg("--no-mmap").arg("foo").arg("-gfile");
+    cmd.assert_err();
+});
+
+rgtest!(binary_quit_mmap, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("file", "foo\x00bar\nfoo\x00baz\n");
+    cmd.arg("--mmap").arg("foo").arg("-gfile");
     cmd.assert_err();
 });
 
