@@ -1,5 +1,5 @@
-use hay::SHERLOCK;
-use util::{Dir, TestCommand, sort_lines};
+use crate::hay::SHERLOCK;
+use crate::util::{Dir, TestCommand, sort_lines};
 
 // See: https://github.com/BurntSushi/ripgrep/issues/16
 rgtest!(r16, |dir: Dir, mut cmd: TestCommand| {
@@ -562,3 +562,179 @@ rgtest!(r900, |dir: Dir, mut cmd: TestCommand| {
 
     cmd.arg("-fpat").arg("sherlock").assert_err();
 });
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1064
+rgtest!(r1064, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("input", "abc");
+    eqnice!("input:abc\n", cmd.arg("a(.*c)").stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1174
+rgtest!(r1098, |dir: Dir, mut cmd: TestCommand| {
+    dir.create_dir(".git");
+    dir.create(".gitignore", "a**b");
+    dir.create("afoob", "test");
+    cmd.arg("test").assert_err();
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1130
+rgtest!(r1130, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("foo", "test");
+    eqnice!(
+        "foo\n",
+        cmd.arg("--files-with-matches").arg("test").arg("foo").stdout()
+    );
+
+    let mut cmd = dir.command();
+    eqnice!(
+        "foo\n",
+        cmd.arg("--files-without-match").arg("nada").arg("foo").stdout()
+    );
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1159
+rgtest!(r1159_invalid_flag, |_: Dir, mut cmd: TestCommand| {
+    cmd.arg("--wat").assert_exit_code(2);
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1159
+rgtest!(r1159_exit_status, |dir: Dir, _: TestCommand| {
+    dir.create("foo", "test");
+
+    // search with a match gets 0 exit status.
+    let mut cmd = dir.command();
+    cmd.arg("test").assert_exit_code(0);
+
+    // search with --quiet and a match gets 0 exit status.
+    let mut cmd = dir.command();
+    cmd.arg("-q").arg("test").assert_exit_code(0);
+
+    // search with a match and an error gets 2 exit status.
+    let mut cmd = dir.command();
+    cmd.arg("test").arg("no-file").assert_exit_code(2);
+
+    // search with a match in --quiet mode and an error gets 0 exit status.
+    let mut cmd = dir.command();
+    cmd.arg("-q").arg("test").arg("foo").arg("no-file").assert_exit_code(0);
+
+    // search with no match gets 1 exit status.
+    let mut cmd = dir.command();
+    cmd.arg("nada").assert_exit_code(1);
+
+    // search with --quiet and no match gets 1 exit status.
+    let mut cmd = dir.command();
+    cmd.arg("-q").arg("nada").assert_exit_code(1);
+
+    // search with no match and an error gets 2 exit status.
+    let mut cmd = dir.command();
+    cmd.arg("nada").arg("no-file").assert_exit_code(2);
+
+    // search with no match in --quiet mode and an error gets 2 exit status.
+    let mut cmd = dir.command();
+    cmd.arg("-q").arg("nada").arg("foo").arg("no-file").assert_exit_code(2);
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1163
+rgtest!(r1163, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("bom.txt", "\u{FEFF}test123\ntest123");
+    eqnice!(
+        "bom.txt:test123\nbom.txt:test123\n",
+        cmd.arg("^test123").stdout()
+    );
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1164
+rgtest!(r1164, |dir: Dir, mut cmd: TestCommand| {
+    dir.create_dir(".git");
+    dir.create(".gitignore", "myfile");
+    dir.create("MYFILE", "test");
+
+    cmd.arg("--ignore-file-case-insensitive").arg("test").assert_err();
+    eqnice!(
+        "MYFILE:test\n",
+        cmd.arg("--no-ignore-file-case-insensitive").stdout()
+    );
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1173
+rgtest!(r1173, |dir: Dir, mut cmd: TestCommand| {
+    dir.create_dir(".git");
+    dir.create(".gitignore", "**");
+    dir.create("foo", "test");
+    cmd.arg("test").assert_err();
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1174
+rgtest!(r1174, |dir: Dir, mut cmd: TestCommand| {
+    dir.create_dir(".git");
+    dir.create(".gitignore", "**/**/*");
+    dir.create_dir("a");
+    dir.create("a/foo", "test");
+    cmd.arg("test").assert_err();
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1176
+rgtest!(r1176_literal_file, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("patterns", "foo(bar\n");
+    dir.create("test", "foo(bar");
+
+    eqnice!(
+        "foo(bar\n",
+        cmd.arg("-F").arg("-f").arg("patterns").arg("test").stdout()
+    );
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1176
+rgtest!(r1176_line_regex, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("patterns", "foo\n");
+    dir.create("test", "foobar\nfoo\nbarfoo\n");
+
+    eqnice!(
+        "foo\n",
+        cmd.arg("-x").arg("-f").arg("patterns").arg("test").stdout()
+    );
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1203
+rgtest!(r1203_reverse_suffix_literal, |dir: Dir, _: TestCommand| {
+    dir.create("test", "153.230000\n");
+
+    let mut cmd = dir.command();
+    eqnice!("153.230000\n", cmd.arg(r"\d\d\d00").arg("test").stdout());
+
+    let mut cmd = dir.command();
+    eqnice!("153.230000\n", cmd.arg(r"\d\d\d000").arg("test").stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1223
+rgtest!(r1223_no_dir_check_for_default_path, |dir: Dir, mut cmd: TestCommand| {
+    dir.create_dir("-");
+    dir.create("a.json", "{}");
+    dir.create("a.txt", "some text");
+
+    eqnice!(
+        "a.json\na.txt\n",
+        sort_lines(&cmd.arg("a").pipe(b"a.json\na.txt"))
+    );
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1259
+rgtest!(r1259_drop_last_byte_nonl, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("patterns-nonl", "[foo]");
+    dir.create("patterns-nl", "[foo]\n");
+    dir.create("test", "fz");
+
+    eqnice!("fz\n", cmd.arg("-f").arg("patterns-nonl").arg("test").stdout());
+    cmd = dir.command();
+    eqnice!("fz\n", cmd.arg("-f").arg("patterns-nl").arg("test").stdout());
+});
+
+// See: https://github.com/BurntSushi/ripgrep/issues/1334
+rgtest!(r1334_crazy_literals, |dir: Dir, mut cmd: TestCommand| {
+    dir.create("patterns", &"1.208.0.0/12\n".repeat(40));
+    dir.create("corpus", "1.208.0.0/12\n");
+    eqnice!(
+        "1.208.0.0/12\n",
+        cmd.arg("-Ff").arg("patterns").arg("corpus").stdout()
+    );
+});
